/**
 * \mainpage
 * 	This is a small "how to use"-example of an simple implementation of a command-interface
 * 	using the internal UART of the AVRMega32 to receive commands from an terminal (e.g. TeraTerm, Hyperterminal) 
 *
 * \section Explanation command-interface
 * 	This example creates three commands using the function
 *	\li cmd_interface_add_cmd("help",	&command_0);
 *	\li cmd_interface_add_cmd("c1",		&command_1);
 *	\li cmd_interface_add_cmd("test",	&command_2);
 *
 * \section Setup Com-Interface
 * \li Baudrate:	9600
 * \li Databits:	8
 * \li Parity:		None
 * \li Stopbits:	1
 * \li FlowContr.:	none
 *
 * \section Setup AVRMega M32
 * 
 * \li Clock:		16Mhz
 * \li Fuses:		....
 */

#include "ringbuffer.h"
#include "AVR_USART.h"
#include "cmd_interface.h"
#include "string_addon.h"
#include <ctype.h>


/* Function Prototypes
   ************************************************/
	void init_ports();
	void avr_init_usart( void );

	/* command functions */
    uint8_t command_0(int8_t argc, char *argval[]);
    uint8_t command_1(int8_t argc, char *argval[]);
    uint8_t command_2(int8_t argc, char *argval[]);
    uint8_t command_3(int8_t argc, char *argval[]);
    uint8_t command_4(int8_t argc, char *argval[]);
    uint8_t command_5(int8_t argc, char *argval[]);
    uint8_t command_6(int8_t argc, char *argval[]);
    uint8_t command_7(int8_t argc, char *argval[]);
    uint8_t command_8(int8_t argc, char *argval[]);
    uint8_t command_9(int8_t argc, char *argval[]);

/* Defines
   ************************************************/
	#define _terminal_startup_message "\n The AVR is ready to receive your commands"

/* Global variables
   ************************************************/
	_RINGBUFFER_T glob_usart_receive_buffer;


/* redirect stdout >> USART
   ************************************************/

	/* define function for stdout */
	int stdout_usart( char c, FILE *stream ) { 	avr_usart_putc(c); 	return 0; 	}
	static FILE mystdout = FDEV_SETUP_STREAM( stdout_usart, NULL, _FDEV_SETUP_WRITE );
	/* eigenen Stream erzeugen
		define FDEV_SETUP_STREAM  (	put,  	- Funktionpointer auf Schreibfunktion
									get,    - Funktionspointer auf Lesefunktion
									rwflag  - Schreib/Lesezugriff
									)
	*/


/**	*
  	*	MAIN
  	*
	******************************************************************/

	char str_rcv_usart[80]="";

int main(void){
	int8_t i8_tmp=0;

	/* redirect stdout into our own stream */
	stdout = &mystdout;

	/* Initialisation */
	init_ports();
	ringbuffer_t_init(&glob_usart_receive_buffer);
    avr_usart_init_9600_8_1_N_Asyn();
	cmd_interface_init();

	/* add command-functions to command-interface */
	cmd_interface_add_cmd("help",	&command_0);
	cmd_interface_add_cmd("c1",		&command_1);
	cmd_interface_add_cmd("test",	&command_2);


//	sprintf(str_rcv_usart,"test op1, op2, op3%c",toascii(13));
//				i8_tmp=execute_cmd_interface(&str_rcv_usart[0]);
	
	/* enable interrupts */
	sei();

	printf(_terminal_startup_message);
	while(1){
		printf("\n\rcmd>");
		do {
			i8_tmp=ringbuffer_flush(&glob_usart_receive_buffer, (uint8_t *)&str_rcv_usart[0]);
			if (0!=i8_tmp){
				str_rcv_usart[i8_tmp]=0; /* Set EOS */
				i8_tmp=execute_cmd_interface(&str_rcv_usart[0]);
			}
		}while(0==i8_tmp);
		if (i8_tmp<0) {
			printf ("\n\r Error: %x",i8_tmp);
		}
	}
	printf("End des Programms - wenn das kommt ist was schief gelaufen ;)");

	return 0;
} /* int main (void) */

/**	*
  	*	ISR
  	*
	******************************************************************/

	/* USART IRQ */
	ISR (USART_RXC_vect) {
		ringbuffer_add(&glob_usart_receive_buffer,avr_usart_getc_isr());
	}


/**	*
  	*	Functions
  	*
	******************************************************************/

void init_ports(){
	/* Define directions for port pins ... 0 = Input; 1 = Output*/
		DDRA = (0<<DDA7)|(0<<DDA6)|(0<<DDA5)|(0<<DDA4)|(0<<DDA3)|(0<<DDA2)|(0<<DDA1)|(0<<DDA0);
		PORTA = (0<<PA7)|(0<<PA6)|(0<<PA5)|(0<<PA4)|(0<<PA3)|(0<<PA2)|(0<<PA1)|(0<<PA0); /* 0 = no PullUps */
	/* Define directions for port pins ... 0 = Input; 1 = Output*/
		DDRB = (0<<DDB7)|(0<<DDB6)|(0<<DDB5)|(0<<DDB4)|(0<<DDB3)|(0<<DDB2)|(0<<DDB1)|(0<<DDB0);
		PORTB = (0<<PB7)|(0<<PB6)|(0<<PB5)|(0<<PB4)|(0<<PB3)|(0<<PB2)|(0<<PB1)|(0<<PB0); /* 0 = no PullUps */
	/* Define directions for port pins ... 0 = Input; 1 = Output*/
		DDRC = (0<<DDC7)|(0<<DDC6)|(0<<DDC5)|(0<<DDC4)|(0<<DDC3)|(0<<DDC2)|(0<<DDC1)|(0<<DDC0);
		PORTC = (0<<PC7)|(0<<PC6)|(0<<PC5)|(0<<PC4)|(0<<PC3)|(0<<PC2)|(0<<PC1)|(0<<PC0); /* 0 = no PullUps */
	/* Define directions for port pins ... 0 = Input; 1 = Output*/
		DDRD = (0<<DDD7)|(0<<DDD6)|(0<<DDD5)|(0<<DDD4)|(0<<DDD3)|(0<<DDD2)|(0<<DDD1)|(0<<DDD0);
		PORTD = (0<<PD7)|(0<<PD6)|(0<<PD5)|(0<<PD4)|(0<<PD3)|(0<<PD2)|(0<<PD1)|(0<<PD0); /* 0 = no PullUps */
} /* void init_ports() */


/**	*
  	*	Command functions
  	*
	******************************************************************/
uint8_t command_0(int8_t argc, char *argval[]){
	printf("\r\n HELP !");
	printf("\r\n help <enter>");
	printf("\r\n     >> this help text");
	printf("\r\n c1 op1,op2,op3,op4 <enter>");
	printf("\r\n     >> example command 1");
	printf("\r\n test op1,op2,op3,op4 <enter>");
	printf("\r\n     >> example command 1");
	return 0;
}

uint8_t command_1(int8_t argc, char *argval[]){
	uint8_t ui8_cnt;
	printf("\n\r...received command c1, with option\n\r");
	for (ui8_cnt=0; argc>ui8_cnt; ui8_cnt++){
		printf("	%d:%s;\n\r",ui8_cnt,argval[ui8_cnt]);
	}
	return 0;
}

uint8_t command_2(int8_t argc, char *argval[]){
	uint8_t ui8_cnt;
	printf("\n\r...received command test, with option\n\r");
	for (ui8_cnt=0; argc>ui8_cnt; ui8_cnt++){
		printf("	%d:%s;\n\r",ui8_cnt,argval[ui8_cnt]);
	}
	return 0;
}
