/***************************************************************************/
/**
	@file		cmd_interface.h
	@brief		Header-File for cmd_interface.c
	@brief		Simple implementation of an command-interface
	@brief		Target/Compiler: AVR_MEGA / GCC(WinAVR)
	@author		Gerhard Bruenner
	@version	V2.03

	@date		2009.12.29  - GB
				created new function split_command_line() - now using own version of strtok.
				This reduced the operating-time of this function to the half of the previous version.  
				Although changed the function-parameter interface of the user-function to
				uint8_t		cmd_func (int8_t argc, char *argval[])
	@date		2009.11.15  - added function cmd_interface_add_cmd and Backspace-Function - GB
	@date		2009.10.31  - changed the way how commandline is interpret - GB
	@date		2009.10.21  - small updates   - GB
	@date		2009.02.01  - Created         - GB

	@bug		none that I am aware so far

\***************************************************************************/
#ifndef CMD_INTERFACE_H
#define CMD_INTERFACE_H	/**< precompiler help */

/* Include-Dateien
****************************************************************************/
#include <stdio.h>
#include <string.h>
#include "string_addon.h"


/* Defines to PARAMETRISE INTERPRETER
****************************************************************************/
#define _CMD_INTERFACE_MAX_LENGTH			80				/**< max number of signs allowed in a command line !! MUST be lower than 127 char*/
#define _CMD_INTERFACE_MAX_OPERAND_LENGTH	12				/**< max.number of signs per operand in a command line*/
#define _CMD_INTERFACE_MAX_NO_OPERANDS		5				/**< max. no of operands per command line including the command */
#define _CMD_INTERFACE_NO_OF_COMMANDS		10				/**< amount of possible commands */
#define _CMD_INTERFACE_OPERAND_SPLIT_SIGN	_ASCII_COMMA	/**< sign which splits operands of a cmd-line - ascii 44= komma*/
#define _CMD_INTERFACE_COMMAND_SPLIT_SIGN	_ASCII_SPACE	/**< sign which splits command from operands in a cmd-line - ascii 32=Space  */
#define _CMD_INTERFACE_EOL					_ASCII_CR 		/**< End of Line - Sign = carriage return */


/* Defines
****************************************************************************/

	/* ERROR - Bitcodes of the function execute_cmd_interface */
	#define _CMD_INTERFACE_ERR_MASK			0xF8			/**< internal mask for bitcoding */
	#define _CMD_INTERFACE_ERR_OP_2_MANY	0x90			/**< internal mask for error-bit "two many operands"*/
	#define _CMD_INTERFACE_ERR_NO_CMD		0xA0			/**< internal mask for error-bit "no command"*/
	#define _CMD_INTERFACE_ERR_CMD_LINE		0xB0			/**< internal mask for error-bit "command-line too long"*/

/* Typdefinitionen
****************************************************************************/

/** @typedef defines internal type used for store command-string and corrosponding command-function
*/
typedef struct  {
	char 		*cmd;											/**< pointer to string which contains commando */
	uint8_t		(*cmd_func_ptr)	(int8_t argc, char *argval[]);	/**< pointer to function which shall be executed after receiving the commando */
	} cmd_interface_command_table_t;


/* Variables
****************************************************************************/
#ifndef cmd_interface_c
	extern cmd_interface_command_table_t cmd_interface_table[_CMD_INTERFACE_NO_OF_COMMANDS]; /**< internal table used to store the differen combinations of commands and functions */
#endif

/* Funktions-Prototypen
****************************************************************************/
int8_t 	execute_cmd_interface(char *string);
void 	cmd_interface_init();
uint8_t cmd_interface_add_cmd(char (*cmd_string), uint8_t (*cmd_func_ptr)(int8_t argc, char *argval[]) ); /**< @param: argc - number of parameters, argval[]- list of pointers to parameter strings. */

#endif /* cmd_interface_h */
