/***************************************************************************\

  File........: ringbuffer.h
  Author(s)...: Gerhard Brnner
  Target(s)...: AVR_MEGA
  Compiler....: WIN_AVR
  Description.: Routines handling an ringbuffer with varialbe access-length

  	Defines the new datatype:		ringbuffer_t

  Revisions...: V2.02
  2009.01.18  - first shot..
  2009.11.20  - removed toggle bit in struct _RINGBUFFER_T
  				changed name of ringbuffer_t to _RINGBUFFER_T
  2009.12.27  - added global defines for buffer-datatype and access_width

\***************************************************************************/

#ifndef _RINGBUFFER_H
#define _RINGBUFFER_H	/**< precompiler help */

/* Include-Dateien
****************************************************************************/
#include <stdio.h>


/* Defines
****************************************************************************/
#define _RINGBUFFER_MAX_BUFFER_LEN		8 			/**<  define length of databuffer beeing used - This has to be a 2^x number = 2,4,8,16,32,64 ...*/
#define _RINGBUFFER_DATATYPE_CONTENT	uint8_t		/**<  defines the datatype of buffer */
#define _RINGBUFFER_ACCESS_WIDTH		uint8_t		/**<  defines the datatype used as index of the buffer-elements */


/* Typdefinitionen
****************************************************************************/

/** @typedef this typedef define the data structure of the structered datatype RINGBUFFER_T
*/
typedef struct{
	_RINGBUFFER_DATATYPE_CONTENT Data[_RINGBUFFER_MAX_BUFFER_LEN];		/**<  buffer memory 	*/
	_RINGBUFFER_ACCESS_WIDTH WritePos; 									/**<  current (next) write-pos  */
	_RINGBUFFER_ACCESS_WIDTH ReadPos;  									/**<  current (next) read position */
	uint8_t 				 Overflow;									/**<  overflow counter */
	} _RINGBUFFER_T;


/* Funktions-Prototypen
****************************************************************************/
void ringbuffer_t_init( _RINGBUFFER_T *buf );
int8_t ringbuffer_add( _RINGBUFFER_T *buf, _RINGBUFFER_DATATYPE_CONTENT data);
int8_t ringbuffer_read(_RINGBUFFER_T *buf, _RINGBUFFER_DATATYPE_CONTENT *data);
_RINGBUFFER_ACCESS_WIDTH ringbuffer_flush(_RINGBUFFER_T *buf, _RINGBUFFER_DATATYPE_CONTENT *data);

/* Makros/Inlines
****************************************************************************/


#endif /* _RINGBUFFER_H */
